﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace AutoValidate
{
    [DefaultProperty("Text")]
    [ToolboxData("<{0}:WebCustomControl1 runat=server></{0}:WebCustomControl1>")]
    public class FormValidate : Panel
    {
        public InValidateInfo ErrorInfo = new InValidateInfo(string.Empty, string.Empty, string.Empty);

        public override void RenderBeginTag(HtmlTextWriter writer)
        {
            base.RenderBeginTag(writer);
        }

        public override void RenderEndTag(HtmlTextWriter writer)
        {
            base.RenderEndTag(writer);
        }

        protected override void OnLoad(EventArgs e)
        {
            if (!Page.ClientScript.IsClientScriptBlockRegistered("FormValidate"))
            {
                //注意：以下此种script写法，不能添加注释，否则写到前端出错
                string script = 
                "   <script type='text/javascript'>                                                     "+

                "    $(document).ready(function () {                                                    "+
                "          $('input[Validate]').each(function () {                                      "+
                "              var id = $(this).attr('id') + '_Validate_';                              "+
                "              $(\"input[name='\" + id + \"']\").val($(this).attr('Validate'));         "+
                "          });                                                                          "+
                "                                                                                       "+
                "          $('input[InvalidateDesc]').each(function () {                                "+
                "              var id = $(this).attr('id') + '_InvalidateDesc_';                        "+
                "              $(\"input[name='\" + id + \"']\").val($(this).attr('InvalidateDesc'));   "+
                "          });                                                                          "+
                "                                                                                       "+
                "    });                                                                                "+
                "                                                                                       "+
                "   var ErrorInfo = new Object();                                                       "+
                "                                                                                       "+
                "   function ClientFormValidate(divId, errorInfo) {                                     "+
                "       var hasError = false;                                                           "+
                "                                                                                       "+
                "        $('#' + divId + ' input[Validate]').each(function () {                         "+
                "            var clientId = $(this).attr('id');                                         "+
                "            var validateRule = $(this).attr('Validate');                               "+
                "            var InvalidateDesc = $(this).attr('InvalidateDesc');                       "+
                "                                                                                       "+
                "            errorInfo.ControlId = clientId;                                            "+
                "            errorInfo.ValidateRule = validateRule;                                     "+
                "            errorInfo.ErrorDesc = InvalidateDesc;                                      "+
                "                                                                                       "+
                "            if ($(this).val() == '') {                                                 "+
                "                hasError = true;                                                       "+
                "                return false;                                                          "+
                "            }                                                                          "+
                "        });                                                                            "+
                                                                                                        
                "       if (hasError) {                                                                 "+
                "              return false;                                                            "+
                "           }                                                                           "+
                "           return true;                                                                "+
                "  }                                                                                    "+

                "</script>                                                                              ";

                Page.ClientScript.RegisterClientScriptBlock(GetType(), "FormValidate", script);
            }
            base.OnLoad(e);
        }

        protected override void Render(HtmlTextWriter writer)
        {
            string html = string.Empty;
            foreach (Control control in Controls)
            {
                if (control is TextBox)
                {
                    html += string.Format("<input type='hidden' name='{0}' value='{1}' />", control.ID + "_Validate_", "");
                    html += string.Format("<input type='hidden' name='{0}' value='{1}' />", control.ID + "_InvalidateDesc_", "");
                }
            }

            writer.Write(html);
            base.Render(writer);
        }

        #region - 验证 -

        public bool Validate()
        {
            //循环本Panel下的所有服务器控件，对于需要验证的控件得到对应的
            //客户端隐藏域从而得到验证规则
            foreach (Control control in Controls)
            {
                if (control is TextBox)  //此处有待扩展
                {
                    string validateRule = HttpContext.Current.Request.Form[control.ID + "_Validate_"];
                    if (!string.IsNullOrEmpty(validateRule))
                    {
                        //根据得到的验证规则对控件进行验证
                        if (string.IsNullOrEmpty(((TextBox)control).Text))
                        {
                            //验证失败
                            ErrorInfo.ControlId = control.ID;
                            ErrorInfo.ValidateRule = validateRule;

                            string errorDesc = HttpContext.Current.Request.Form[control.ID + "_InvalidateDesc_"];
                            if (!string.IsNullOrEmpty(errorDesc))  //如果用户指定了InvalidateDesc属性
                            {
                                ErrorInfo.ErrorDesc = errorDesc;
                            }
                            return false;
                        };
                    }

                }
            }
            return true;
        }

        public bool ValidateExcept(params Control[] exceptControls)
        {
            foreach (Control control in Controls)
            {
                if (!exceptControls.Contains(control))
                {
                    if (control is TextBox)
                    {
                        string validateRule = HttpContext.Current.Request.Form[control.ID + "_Validate_"];
                        if (!string.IsNullOrEmpty(validateRule))
                        {
                            //根据得到的验证规则对控件进行验证
                            if (string.IsNullOrEmpty(((TextBox)control).Text))
                            {
                                ErrorInfo.ControlId = control.ID;
                                ErrorInfo.ValidateRule = validateRule;
                                return false;
                            };
                        }
                    }
                }
            }
            return true;
        }

        public bool ValidateInclude(params Control[] includeControls)
        {
            foreach (Control control in Controls)
            {
                if (includeControls.Contains(control))
                {
                    if (control is TextBox)
                    {
                        string validateRule = HttpContext.Current.Request.Form[control.ID + "_Validate_"];
                        if (!string.IsNullOrEmpty(validateRule))
                        {
                            //根据得到的验证规则对控件进行验证
                            if (string.IsNullOrEmpty(((TextBox)control).Text))
                            {
                                ErrorInfo.ControlId = control.ID;
                                ErrorInfo.ValidateRule = validateRule;
                                return false;
                            };
                        }
                    }
                }
            }
            return true;
        }

        #endregion
    }

    public class InValidateInfo
    {
        public string ControlId { get; set; }
        public string ValidateRule { get; set; }
        public string ErrorDesc { get; set; }

        public InValidateInfo(string controlId, string validateRule, string errorDesc)
        {
            ControlId = controlId;
            ValidateRule = validateRule;
            ErrorDesc = errorDesc;
        }
    }
}
